@@ -1,17 +1,18 @@
+from typing import Any
+
 from prophet import Prophet
 import pandas as pd
-import os, json, redis
 from sklearn.preprocessing import StandardScaler
 
-from analyser2_hj3415.tsseer.utils import get_raw_data
-from analyser2_hj3415.common.connection import get_redis_client
+from .utils import get_raw_data, redis_cached, judge_trend
+
 
 from utils_hj3415 import setup_logger
 
 mylogger = setup_logger(__name__,'INFO')
 
 
-def prepare_prophet_data(df: pd.DataFrame) -> dict[str, pd.DataFrame | StandardScaler]:
+def prepare_data(df: pd.DataFrame) -> dict[str, pd.DataFrame | StandardScaler]:
     # ── (1) 기본 정리 ────────────────────────────────────
     df.columns = df.columns.get_level_values(0)
     df.index   = df.index.tz_localize(None)
@@ -32,7 +33,7 @@ def prepare_prophet_data(df: pd.DataFrame) -> dict[str, pd.DataFrame | StandardS
     }
 
 
-def run_prophet_forecast(df_scaler_dict: dict[str, object], periods: int = 180) -> pd.DataFrame:
+def run_forecast(df_scaler_dict: dict[str, object], periods: int = 180) -> pd.DataFrame:
     """Prophet 학습 & periods 일 예측 → 역-스케일링해서 반환"""
 
     # ── 0. 필요한 객체 꺼내기 ───────────────────────────
@@ -68,64 +69,76 @@ def run_prophet_forecast(df_scaler_dict: dict[str, object], periods: int = 180)
 
     return pred_df
 
-# ── redis 연결──────────────────────
 
-# ── 캐싱 버전 serve_myprophet_forecast ────────────────────────────
-def cached_prophet_forecast(ticker: str, periods: int = 180) -> dict[str, list]:
+def latest_anomaly_score(
+    df: pd.DataFrame,
+    *,
+    y_col: str = 'actual',
+    lo_col: str = 'lower',
+    up_col: str = 'upper',
+    ndigits: int = 4,
+    eps: float = 1e-9
+) -> float | None:
     """
-    1) redis 에 prophet:{ticker} 키가 있으면 그대로 반환
-    2) 없으면 예측 계산 → 캐시에 저장(setex) → 반환
+    최근 값이 예측 구간을 얼마나 벗어났는지 비율로 반환.
+    - upper 초과 → 양수
+    - lower 초과 → 음수
+    - 구간 안 → 0
+    - 계산 불가 → None
     """
-    redis_cli = get_redis_client()
-    ttl = int(os.getenv('REDIS_EXPIRE_TIME_H', 12)) * 60 * 60
+    valid = df.dropna(subset=[y_col, lo_col, up_col])
+    if valid.empty:
+        return None
 
-    cache_key = f"prophet:{ticker.lower()}"
+    y, lo, up = valid.iloc[-1][[y_col, lo_col, up_col]]
 
-    # 1. 캐시 조회 ---------------------------------------------------
-    try:
-        if (raw := redis_cli.get(cache_key)):
-            mylogger.info(f"cache hit {cache_key}")
-            return json.loads(raw)
-    except redis.RedisError as e:
-        mylogger.warning(f"Redis GET fail: {e}")
+    width = max(up - lo, eps)
 
-    # 2. 캐시 미스 → 예측 계산 --------------------------------------
-    mylogger.info(f"Prophet cache miss: {cache_key} — start forecasting")
+    if y > up:
+        score = (y - up) / width  # 양수
+    elif y < lo:
+        score = -(lo - y) / width  # 음수
+    else:
+        score = 0.0
 
-    # 2-1 데이터 준비
+    return round(score, ndigits)
+
+
+@redis_cached(prefix='prophet')
+def prophet_forecast(ticker: str, refresh: bool = False, cache_only: bool = False) -> dict[str, Any]:
+    """
+    Pure 도메인 로직: (1)데이터 준비 → (2)예측 → (3)직렬화 dict 반환
+    Redis 캐싱은 데코레이터가 담당.
+    """
     df = get_raw_data(ticker)
-    df_scaler_dict = prepare_prophet_data(df)
+    df_scaler_dict = prepare_data(df)
+    past_and_forecast = run_forecast(df_scaler_dict)
 
-    # 2-2 예측
-    past_and_forecast_df = run_prophet_forecast(df_scaler_dict, periods)
-
-    # 2-3 직렬화용 머지
     prepared_df: pd.DataFrame = df_scaler_dict['prepared_df']
     merged = (
         prepared_df[['ds', 'y']]
         .merge(
-            past_and_forecast_df[['ds', 'yhat', 'yhat_lower', 'yhat_upper']],
+            past_and_forecast[['ds', 'yhat', 'yhat_lower', 'yhat_upper']],
             on='ds', how='outer'
         )
         .sort_values('ds')
     )
 
-    result: dict[str, list] = {
+    # ------------- (2) 추세 평가 -------------
+    fcst_vals = merged.loc[merged['yhat'].notna(), 'yhat'].values
+    trend = judge_trend(fcst_vals)  # "상승" | "하락" | "횡보" | 미정
+
+    # 3-1) anomaly score 계산
+    anomaly = latest_anomaly_score(merged, y_col='y', lo_col='yhat_lower', up_col='yhat_upper')
+
+    return {
         'ds'      : merged['ds'].dt.strftime('%Y-%m-%d').tolist(),
         'actual'  : merged['y'].where(merged['y'].notna()).tolist(),
         'forecast': merged['yhat'].round(2).tolist(),
         'lower'   : merged['yhat_lower'].round(2).tolist(),
         'upper'   : merged['yhat_upper'].round(2).tolist(),
+        'trend': trend,
+        "anomaly_score": anomaly
     }
 
-    # 3. 캐시에 저장 -------------------------------------------------
-    try:
-        redis_cli.setex(cache_key, ttl, json.dumps(result))
-        mylogger.info(f"cache save {cache_key} (ttl={ttl})")
-    except redis.RedisError as e:
-        mylogger.warning(f"Redis SETEX fail: {e}")
-
-    return result
-
-
 
